"""Webhook Module."""

import hmac
from datetime import datetime, timezone
from hashlib import sha256


class WebhookSignatureVerificationError(ValueError):
    """Error raised when webhook signature verification fails."""

    pass


class WebhookNoTimestampError(WebhookSignatureVerificationError):
    """Error raised when the timestamp is missing from the signature."""

    def __init__(self) -> None:
        """Initialize WebhookNoTimestampError."""
        super().__init__("No timestamp found")


class WebhookNoSignatureError(WebhookSignatureVerificationError):
    """Error raised when a signature matching the scheme is missing."""

    def __init__(self) -> None:
        """Initialize WebhookNoSignatureError."""
        super().__init__("No signatures found")


class WebhookSignatureMismatchError(WebhookSignatureVerificationError):
    """Error raised when the signature does not match the expected one."""

    def __init__(self) -> None:
        """Initialize WebhookSignatureMismatchError."""
        super().__init__("No signatures found matching the expected signature")


class WebhookTimestampOutsideToleranceError(WebhookSignatureVerificationError):
    """Error raised when timestamp is outside the tolerance window."""

    def __init__(self) -> None:
        """Initialize WebhookTimestampOutsideToleranceError."""
        super().__init__("Timestamp outside the tolerance window")


def verify(
    payload: bytes,
    signature: str,
    secret: str,
    tolerance: int = 300,
    scheme: str = "v1",
) -> None:
    """Verify webhook signature.

    Args:
        payload: Raw webhook request body.
        signature: The signature to verify.
        secret: The webhook secret.
        tolerance: Maximum allowed age of the timestamp in seconds. Defaults to 300.
        scheme: Key for signatures in the signature. Defaults to "v1".

    Raises:
        WebhookNoTimestampError: Error raised when the timestamp is missing from the
            signature.
        WebhookNoSignatureError: Error raised when a signature matching the scheme is
            missing.
        WebhookSignatureMismatchError: Error raised when the signature does not match
            the expected one.
        WebhookTimestampOutsideToleranceError: Error raised when timestamp is outside
            the tolerance window.
    """
    timestamp, signature = _parse_signature(signature, scheme)

    _verify_timestamp(timestamp, tolerance)

    signed_payload = b"%d." % timestamp + payload

    expected_signature = _compute_signature(signed_payload, secret)

    _compare(expected_signature, signature)


def _parse_signature(signature: str, scheme: str) -> tuple[int, str]:
    """Parse the signature to extract timestamp and signature for the schema.

    Args:
        signature: The signature to parse.
        scheme: Key for signatures in the signature.

    Returns:
        Tuple containing timestamp and signature.

    Raises:
        WebhookNoTimestampError: Error raised when the timestamp is missing from the
            signature.
        WebhookNoSignatureError: Error raised when a signature matching the scheme is
            missing.
    """
    list_items = [item.split("=", 1) for item in signature.split(",")]

    timestamp_str = next(
        (item[1] for item in list_items if len(item) == 2 and item[0] == "t"),
        None,
    )
    if timestamp_str is None:
        raise WebhookNoTimestampError()

    matchedSignature = next(
        (item[1] for item in list_items if len(item) == 2 and item[0] == scheme), None
    )

    if not matchedSignature:
        raise WebhookNoSignatureError()

    return int(timestamp_str), matchedSignature


def _compute_signature(payload: bytes, secret: str) -> str:
    """Compute signature using payload and webhook secret.

    Args:
        payload: Payload bytes from the webhook request body.
        secret: The webhook secret.

    Returns:
        Computed signature.
    """
    mac = hmac.new(secret.encode("utf-8"), msg=payload, digestmod=sha256)
    return mac.hexdigest()


def _compare(expected_signature: str, signature: str) -> None:
    """Compare signature with the expected signature.

    Args:
        expected_signature: The expected signature.
        signature: The signature.

    Raises:
        WebhookSignatureMismatchError: Error raised when the signature does not match
            the expected one.
    """
    if hmac.compare_digest(expected_signature, signature):
        return None
    raise WebhookSignatureMismatchError()


def _verify_timestamp(timestamp: int, tolerance: int) -> None:
    """Verify that the webhook timestamp is within the tolerance window.

    Args:
        timestamp: Timestamp from the webhook signature.
        tolerance: Maximum allowed age of the timestamp in seconds.

    Raises:
        WebhookTimestampOutsideToleranceError: Error raised when timestamp is outside
            the tolerance window.
    """
    now_utc = datetime.now(timezone.utc).timestamp()
    if timestamp < now_utc - tolerance or timestamp > now_utc + tolerance:
        raise WebhookTimestampOutsideToleranceError()



"""Package initializer for WriftAI Python Client."""

from wriftai._client import Client, ClientOptions
from wriftai._webhook import (
    WebhookNoSignatureError,
    WebhookNoTimestampError,
    WebhookSignatureMismatchError,
    WebhookSignatureVerificationError,
    WebhookTimestampOutsideToleranceError,
)
from wriftai._webhook import (
    verify as verify_webhook,
)
from wriftai.pagination import PaginationOptions

__all__ = [
    "Client",
    "ClientOptions",
    "PaginationOptions",
    "WebhookNoSignatureError",
    "WebhookNoTimestampError",
    "WebhookSignatureMismatchError",
    "WebhookSignatureVerificationError",
    "WebhookTimestampOutsideToleranceError",
    "verify_webhook",
]



from unittest.mock import AsyncMock, Mock, patch

import pytest

from wriftai.model_categories import ModelCategories
from wriftai.pagination import PaginationOptions


@patch("wriftai.model_categories.PaginatedResponse")
@pytest.mark.asyncio
@pytest.mark.parametrize("async_flag", [True, False])
async def test_listing(mock_paginated_response: Mock, async_flag: bool) -> None:
    test_response = {"key": "value"}
    pagination_options = PaginationOptions({"cursor": "abc123", "page_size": 50})

    if async_flag:
        mock_api = AsyncMock()
        mock_api.async_request.return_value = test_response
    else:
        mock_api = Mock()
        mock_api.request.return_value = test_response

    model_category = ModelCategories(api=mock_api)

    if async_flag:
        result = await model_category.async_list(pagination_options=pagination_options)
        mock_api.async_request.assert_called_once_with(
            method="GET",
            path="/model_categories",
            params=pagination_options,
        )
    else:
        result = model_category.list(pagination_options=pagination_options)
        mock_api.request.assert_called_once_with(
            method="GET",
            path="/model_categories",
            params=pagination_options,
        )

    mock_paginated_response.assert_called_once_with(**test_response)
    assert result == mock_paginated_response.return_value



from unittest.mock import Mock

import pytest

from wriftai._resource import Resource
from wriftai.models import ModelsResource
from wriftai.predictions import Predictions


class MockAPIResource(Resource):
    """Concrete subclass for testing the abstract Resource."""

    pass


def test_api_resource() -> None:
    mock_api = Mock()
    resource = MockAPIResource(api=mock_api)
    assert resource._api == mock_api


@pytest.mark.parametrize(
    "invalid_model",
    ["invalidformat", "/modelname", "owner/", "owner/name:"],
)
def test__parse_identifier_raises_error(invalid_model: str) -> None:
    mock_api = Mock()
    prediction = Predictions(api=mock_api)

    with pytest.raises(ValueError) as e:
        prediction._parse_identifier(identifier=invalid_model)

    assert str(e.value) == prediction._ERROR_MSG_INVALID_IDENTIFIER


def test__parse_model_identifier_raises_error() -> None:
    mock_api = Mock()
    model = ModelsResource(api=mock_api)

    with pytest.raises(ValueError) as e:
        model._parse_model_identifier(identifier="modelowner/modelname:2")

    assert str(e.value) == model._ERROR_MSG_INVALID_MODEL_IDENTIFIER



"""Common types used across the WriftAI package."""

import sys
from collections.abc import Mapping
from enum import Enum
from typing import Any, TypeAlias, TypedDict, Union

__all__ = [
    "BaseUser",
    "JsonValue",
    "Model",
    "ModelCategory",
    "ModelVersion",
    "ModelVersionWithDetails",
    "ModelVisibility",
    "NotRequired",
    "SchemaIO",
    "Schemas",
    "SortDirection",
    "StrEnum",
    "User",
    "UserWithDetails",
]

JsonValue: TypeAlias = Union[
    list["JsonValue"],
    Mapping[str, "JsonValue"],
    str,
    bool,
    int,
    float,
    None,
]
"""A JSON-compatible value."""


class _FallbackStrEnum(str, Enum):
    """Fallback StrEnum for python 3.10 version."""

    def __str__(self) -> str:
        """Return the enum value as string."""
        return str(self.value)


if sys.version_info >= (3, 11):
    from enum import StrEnum
    from typing import NotRequired
else:
    from enum import Enum

    from typing_extensions import NotRequired

    StrEnum = _FallbackStrEnum


class BaseUser(TypedDict):
    """Represents a user with basic details."""

    id: str
    """Unique identifier of the user."""
    username: str
    """The username of the user."""
    avatar_url: str
    """URL of the user's avatar."""


class User(BaseUser):
    """Represents a user."""

    name: str | None
    """The name of the user."""
    bio: str | None
    """The biography of the user."""
    location: str | None
    """Location of the user."""
    company: str | None
    """Company the user is associated with."""
    created_at: str
    """Timestamp when the user joined WriftAI."""
    updated_at: str | None
    """Timestamp when the user was last updated."""


class UserWithDetails(User):
    """Represents a user with details."""

    urls: list[str] | None
    """Personal or professional website URLs."""


class SchemaIO(TypedDict):
    """Represents input and output schemas."""

    input: dict[str, Any]
    """Schema for input, following JSON Schema Draft 2020-12 standards."""
    output: dict[str, Any]
    """Schema for output, following JSON Schema Draft 2020-12 standards."""


class Schemas(TypedDict):
    """Represents schemas of a model version."""

    prediction: SchemaIO
    """The input and output schemas for a prediction."""


class ModelVersion(TypedDict):
    """Represents a model version."""

    number: int
    """The number of the model version."""
    release_notes: str
    """Information about changes such as new features,bug fixes,
        or optimizations in this model version."""
    created_at: str
    """The time when the model version was created."""
    container_image_digest: str
    """A sha256 hash digest of the model version's container image."""


class ModelVersionWithDetails(ModelVersion):
    """Represents a model version with details."""

    schemas: Schemas
    """The schemas of the model version."""

    openai_chat_completions: bool
    """Represents whether this version supports the
    OpenAI compatible chat completions endpoint."""

    openai_embeddings: bool
    """Represents whether this version supports the
    OpenAI compatible embeddings endpoint."""

    openai_images_generations: bool
    """Represents whether this version supports the
    OpenAI compatible image generation endpoint."""

    anthropic_messages: bool
    """Represents whether this version supports the
    Anthropic compatible messages endpoint."""


class Hardware(TypedDict):
    """Represents a hardware item."""

    name: str
    """The name of the hardware."""

    identifier: str
    """The identifier of the hardware."""


class SortDirection(StrEnum):
    """Enumeration of possible sorting directions."""

    ASC = "asc"
    DESC = "desc"


class ModelVisibility(StrEnum):
    """Model visibility states."""

    private = "private"
    public = "public"


class ModelCategory(TypedDict):
    """Represents a model category."""

    name: str
    """Name of the model category."""
    slug: str
    """Slug of the model category."""


class Model(TypedDict):
    """Represents a model."""

    id: str
    """The unique identifier of the model."""
    name: str
    """ The name of the model."""
    created_at: str
    """The time when the model was created."""
    visibility: ModelVisibility
    """The visibility of the model."""
    description: str | None
    """Description of the model."""
    updated_at: str | None
    """The time when the model was updated."""
    owner: BaseUser
    """The details of the owner of the model."""
    hardware: Hardware
    """The hardware used by the model."""
    predictions_count: int
    """The total number of predictions created across all versions
        of the model."""
    categories: list[ModelCategory]
    """The categories associated with the model."""



from unittest.mock import AsyncMock, Mock, patch

import pytest

from wriftai.model_versions import CreateModelVersionParams, ModelVersions
from wriftai.pagination import PaginationOptions


def test_get() -> None:
    mock_api = Mock()

    model_version = ModelVersions(api=mock_api)
    model_owner = "deepseek-ai"
    model_name = "deepseek-r1"
    number = 1

    result = model_version.get(identifier=f"{model_owner}/{model_name}:{number}")

    mock_api.request.assert_called_once_with(
        "GET",
        f"{model_version._MODELS_API_PREFIX}/{model_owner}/{model_name}{model_version._MODEL_VERSIONS_PATH}/{number}",
    )
    assert result == mock_api.request.return_value


@pytest.mark.asyncio
async def test_async_get() -> None:
    mock_api = Mock()
    mock_api.async_request = AsyncMock()

    model_version = ModelVersions(api=mock_api)
    model_owner = "deepseek-ai"
    model_name = "deepseek-r1"
    number = 1

    result = await model_version.async_get(
        identifier=f"{model_owner}/{model_name}:{number}"
    )

    mock_api.async_request.assert_called_once_with(
        "GET",
        f"{model_version._MODELS_API_PREFIX}/{model_owner}/{model_name}{model_version._MODEL_VERSIONS_PATH}/{number}",
    )
    assert result == mock_api.async_request.return_value


@patch("wriftai.model_versions.PaginatedResponse")
def test_list(mock_paginated_response: Mock) -> None:
    mock_api = Mock()
    test_response = {"key": "value"}
    mock_api.request.return_value = test_response

    versions = ModelVersions(api=mock_api)
    pagination_options = PaginationOptions({"cursor": "abc123", "page_size": 50})
    model_owner = "abc"
    model_name = "textgenerator"

    result = versions.list(
        identifier=f"{model_owner}/{model_name}",
        pagination_options=pagination_options,
    )
    path = (
        f"{versions._MODELS_API_PREFIX}/{model_owner}"
        f"/{model_name}{versions._MODEL_VERSIONS_PATH}"
    )
    mock_api.request.assert_called_once_with(
        method="GET", path=path, params=pagination_options
    )
    mock_paginated_response.assert_called_once_with(**test_response)
    assert result == mock_paginated_response.return_value


@patch("wriftai.model_versions.PaginatedResponse")
@pytest.mark.asyncio
async def test_async_list(mock_paginated_response: Mock) -> None:
    mock_api = AsyncMock()
    test_response = {"key": "value"}
    mock_api.async_request.return_value = test_response

    versions = ModelVersions(api=mock_api)
    pagination_options = PaginationOptions({"cursor": "abc123", "page_size": 50})
    model_owner = "abc"
    model_name = "textgenerator"

    result = await versions.async_list(
        identifier=f"{model_owner}/{model_name}",
        pagination_options=pagination_options,
    )

    path = (
        f"{versions._MODELS_API_PREFIX}/{model_owner}"
        f"/{model_name}{versions._MODEL_VERSIONS_PATH}"
    )
    mock_api.async_request.assert_called_once_with(
        method="GET", path=path, params=pagination_options
    )
    mock_paginated_response.assert_called_once_with(**test_response)
    assert result == mock_paginated_response.return_value


def test_delete() -> None:
    mock_api = Mock()

    model_version = ModelVersions(api=mock_api)
    model_owner = "deepseek-ai"
    model_name = "deepseek-r1"
    number = 1

    model_version.delete(identifier=f"{model_owner}/{model_name}:{number}")

    mock_api.request.assert_called_once_with(
        "DELETE",
        f"{model_version._MODELS_API_PREFIX}/{model_owner}/{model_name}{model_version._MODEL_VERSIONS_PATH}/{number}",
    )


@pytest.mark.asyncio
async def test_async_delete() -> None:
    mock_api = Mock()
    mock_api.async_request = AsyncMock()

    model_version = ModelVersions(api=mock_api)
    model_owner = "deepseek-ai"
    model_name = "deepseek-r1"
    number = 1

    await model_version.async_delete(identifier=f"{model_owner}/{model_name}:{number}")

    mock_api.async_request.assert_called_once_with(
        "DELETE",
        f"{model_version._MODELS_API_PREFIX}/{model_owner}/{model_name}{model_version._MODEL_VERSIONS_PATH}/{number}",
    )


def test_create() -> None:
    mock_api = Mock()

    model = ModelVersions(api=mock_api)

    model_owner = "abc"
    model_name = "textgenerator"
    options: CreateModelVersionParams = {
        "release_notes": "Initial release with basic features",
        "container_image_digest": "sha256:" + "a" * 64,
        "schemas": {
            "prediction": {
                "input": {"key1": "value1", "key2": 123},
                "output": {"result": True, "message": "Success"},
            }
        },
        "anthropic_messages": False,
        "openai_chat_completions": True,
        "openai_embeddings": False,
        "openai_images_generations": False,
    }

    result = model.create(
        identifier=f"{model_owner}/{model_name}",
        options=options,
    )
    path = (
        f"{model._MODELS_API_PREFIX}/{model_owner}"
        f"/{model_name}{model._MODEL_VERSIONS_PATH}"
    )
    mock_api.request.assert_called_once_with(
        "POST",
        path,
        body=options,
    )

    assert result == mock_api.request.return_value


@pytest.mark.asyncio
async def test_async_create() -> None:
    mock_api = Mock()
    mock_api.async_request = AsyncMock()

    model = ModelVersions(api=mock_api)

    model_owner = "abc"
    model_name = "textgenerator"
    options: CreateModelVersionParams = {
        "release_notes": "Initial release with basic features",
        "container_image_digest": "sha256:" + "a" * 64,
        "schemas": {
            "prediction": {
                "input": {"key1": "value1", "key2": 123},
                "output": {"result": True, "message": "Success"},
            }
        },
        "anthropic_messages": False,
        "openai_chat_completions": True,
        "openai_embeddings": False,
        "openai_images_generations": False,
    }

    result = await model.async_create(
        identifier=f"{model_owner}/{model_name}",
        options=options,
    )

    path = (
        f"{model._MODELS_API_PREFIX}/{model_owner}"
        f"/{model_name}{model._MODEL_VERSIONS_PATH}"
    )

    mock_api.async_request.assert_called_once_with(
        "POST",
        path,
        body=options,
    )

    assert result == mock_api.async_request.return_value


def test__parse_model_version_identifier_raises_error() -> None:
    mock_api = Mock()
    model_version = ModelVersions(api=mock_api)

    with pytest.raises(ValueError) as e:
        model_version._parse_model_version_identifier(identifier="modelowner/modelname")

    assert str(e.value) == model_version._ERROR_MSG_INVALID_MODEL_VERSION_IDENTIFIER



"""Model categories module."""

from typing import Optional

from wriftai._resource import Resource
from wriftai.common_types import ModelCategory
from wriftai.pagination import PaginatedResponse, PaginationOptions


class ModelCategoryWithDetails(ModelCategory):
    """Represents a model category with details."""

    description: str
    """Description of the model category."""


class ModelCategories(Resource):
    """Resource for operations related to model categories."""

    def list(
        self, pagination_options: Optional[PaginationOptions]
    ) -> PaginatedResponse[ModelCategoryWithDetails]:
        """List model categories.

        Args:
            pagination_options: Optional settings to control pagination behavior.

        Returns:
            Paginated response containing model category items and navigation metadata.
        """
        response = self._api.request(
            method="GET", params=pagination_options, path="/model_categories"
        )

        # The response will always match the PaginatedResponse structure,
        # but static type checkers may not infer this correctly.
        # Hence, we ignore the argument type warning.
        return PaginatedResponse(**response)  # type:ignore[arg-type]

    async def async_list(
        self, pagination_options: Optional[PaginationOptions]
    ) -> PaginatedResponse[ModelCategoryWithDetails]:
        """List model categories.

        Args:
            pagination_options: Optional settings to control pagination behavior.

        Returns:
            Paginated response containing model category items and navigation metadata.
        """
        response = await self._api.async_request(
            method="GET", params=pagination_options, path="/model_categories"
        )

        # The response will always match the PaginatedResponse structure,
        # but static type checkers may not infer this correctly.
        # Hence, we ignore the argument type warning.
        return PaginatedResponse(**response)  # type:ignore[arg-type]



import json
from typing import Callable, Optional

import httpx
import pytest
from respx import Router

from tests.integration.conftest import Route
from wriftai import Client, ClientOptions
from wriftai.common_types import ModelVisibility, SortDirection
from wriftai.models import (
    CreateModelParams,
    ModelPaginationOptions,
    ModelsSortBy,
    UpdateModelParams,
)
from wriftai.pagination import PaginatedResponse, PaginationOptions


@pytest.mark.asyncio
@pytest.mark.parametrize("async_flag", [True, False])
@pytest.mark.parametrize("owner", [None, "test_username"])
async def test_list(
    mock_router: Callable[..., Router], async_flag: bool, owner: Optional[str]
) -> None:
    expected_owner = owner or "test_username"
    path = f"/models/{expected_owner}" if owner else "/models"
    expected_cursor = "abc123"
    expected_page_size = "10"
    test_category_slugs = ["slug-1", "slug-2", "slug-3"]

    expected_json = {
        "items": [
            {
                "id": "123e4567-e89b-12d3-a456-426614174000",
                "name": "test_model_name",
                "created_at": "2025-08-01T10:00:00Z",
                "visibility": ModelVisibility.public,
                "description": "A computer vision model.",
                "updated_at": "2025-08-15T14:30:00Z",
                "owner": {
                    "id": "user-001",
                    "username": expected_owner,
                    "avatar_url": "https://example.com/avatar.png",
                },
                "hardware": {
                    "name": "test hardware name",
                    "identifier": "test hardware identifier",
                },
                "predictions_count": 2048,
                "categories": [
                    {
                        "name": "model category name",
                        "slug": "model category slug",
                    },
                ],
            }
        ],
        "next_cursor": "abc123",
        "previous_cursor": None,
        "next_url": "/models?cursor=abc123",
        "previous_url": None,
    }

    call_count = 0

    def request_assertions(request: httpx.Request) -> None:
        nonlocal call_count
        call_count += 1
        assert request.url.params.get("cursor") == expected_cursor
        assert request.url.params.get("page_size") == expected_page_size
        assert request.url.params.get("sort_by") == ModelsSortBy.CREATED_AT
        assert request.url.params.get("sort_direction") == SortDirection.ASC
        assert request.url.params.get_list("category_slugs") == test_category_slugs
        assert set(request.url.params.keys()).issubset({
            "cursor",
            "page_size",
            "sort_by",
            "sort_direction",
            "category_slugs",
        })

    router = mock_router(
        route=Route(
            method="GET",
            path=path,
            status_code=200,
            json=expected_json,
        ),
        callback=request_assertions,
    )

    client = Client(
        client_options=ClientOptions(
            headers={"Authorization": "test-token"},
            timeout=httpx.Timeout(15),
            transport=httpx.MockTransport(router.handler),
        )
    )

    pagination_options = ModelPaginationOptions({
        "cursor": expected_cursor,
        "page_size": int(expected_page_size),
        "sort_by": ModelsSortBy.CREATED_AT,
        "sort_direction": SortDirection.ASC,
        "category_slugs": test_category_slugs,
    })

    if async_flag:
        response = await client.models.async_list(
            owner=owner, pagination_options=pagination_options
        )
    else:
        response = client.models.list(
            owner=owner, pagination_options=pagination_options
        )

    # expected_json is a dictionary.
    # Although the structure matches the expected fields, static type checkers like mypy
    # may not be able to verify this due to dynamic typing.
    assert response == PaginatedResponse(**expected_json)  # type:ignore[arg-type]
    assert call_count == 1


@pytest.mark.asyncio
@pytest.mark.parametrize("async_flag", [True, False])
async def test_delete(mock_router: Callable[..., Router], async_flag: bool) -> None:
    test_owner = "test_owner"
    test_model_name = "dummy_model"

    call_count = 0

    def request_assertions(request: httpx.Request) -> None:
        nonlocal call_count
        call_count += 1

    router = mock_router(
        route=Route(
            method="DELETE",
            path=f"/models/{test_owner}/{test_model_name}",
            status_code=204,
            json={},
        ),
        callback=request_assertions,
    )

    client = Client(
        client_options=ClientOptions(
            headers={"Authorization": "test-token"},
            timeout=httpx.Timeout(15),
            transport=httpx.MockTransport(router.handler),
        )
    )

    if async_flag:
        await client.models.async_delete(identifier=f"{test_owner}/{test_model_name}")
    else:
        client.models.delete(identifier=f"{test_owner}/{test_model_name}")

    assert call_count == 1


@pytest.mark.asyncio
@pytest.mark.parametrize("async_flag", [True, False])
async def test_get(mock_router: Callable[..., Router], async_flag: bool) -> None:
    test_owner = "test_owner"
    test_model_name = "dummy_model"

    expected_json = {
        "id": "123e4567-e89b-12d3-a456-426614174000",
        "name": test_model_name,
        "created_at": "2025-08-01T10:00:00Z",
        "visibility": ModelVisibility.public,
        "description": "A computer vision model.",
        "updated_at": "2025-08-15T14:30:00Z",
        "source_url": "https://example.com/source",
        "license_url": "https://example.com/license",
        "paper_url": "https://example.com/paper",
        "owner": {
            "id": "user-001",
            "username": test_owner,
            "avatar_url": "https://example.com/avatar.png",
        },
        "latest_version": {
            "number": 1,
            "release_notes": "Initial release.",
            "created_at": "2025-08-10T09:00:00Z",
            "schemas": {
                "prediction": {
                    "input": {"key1": "value1", "key2": 123},
                    "output": {"result": True, "message": "Success"},
                }
            },
            "container_image_digest": "sha256:abc123def456ghi",
        },
        "hardware": {
            "name": "test hardware name",
            "identifier": "test hardware identifier",
        },
        "predictions_count": 2048,
        "overview": "overview of the model",
        "categories": [
            {
                "name": "model category name",
                "slug": "model category slug",
            },
        ],
    }

    call_count = 0

    def request_assertions(request: httpx.Request) -> None:
        nonlocal call_count
        call_count += 1

    router = mock_router(
        route=Route(
            method="GET",
            path=f"/models/{test_owner}/{test_model_name}",
            status_code=200,
            json=expected_json,
        ),
        callback=request_assertions,
    )

    client = Client(
        client_options=ClientOptions(
            headers={"Authorization": "test-token"},
            timeout=httpx.Timeout(15),
            transport=httpx.MockTransport(router.handler),
        )
    )

    if async_flag:
        response = await client.models.async_get(
            identifier=f"{test_owner}/{test_model_name}"
        )
    else:
        response = client.models.get(identifier=f"{test_owner}/{test_model_name}")

    assert response == expected_json
    assert call_count == 1


@pytest.mark.asyncio
@pytest.mark.parametrize("async_flag", [True, False])
async def test_create(mock_router: Callable[..., Router], async_flag: bool) -> None:
    params: CreateModelParams = {
        "name": "test_model",
        "visibility": ModelVisibility.private,
        "hardware_identifier": "test-identifier",
        "description": "Test model description",
        "source_url": "https://example.com/source",
        "license_url": "https://example.com/license",
        "paper_url": "https://example.com/paper",
        "overview": "overview of the model",
        "category_slugs": ["slug-1", "slug-2", "slug-3"],
    }

    expected_json = {
        "id": "123e4567-e89b-12d3-a456-426614174000",
        "name": params["name"],
        "created_at": "2025-08-01T10:00:00Z",
        "visibility": params["visibility"],
        "description": params["description"],
        "updated_at": "2025-08-15T14:30:00Z",
        "source_url": params["source_url"],
        "license_url": params["license_url"],
        "paper_url": params["paper_url"],
        "owner": {
            "id": "user-001",
            "username": "Kunal_Kumar",
            "avatar_url": "https://example.com/avatar.png",
        },
        "latest_version": {
            "number": 1,
            "release_notes": "Initial release.",
            "created_at": "2025-08-10T09:00:00Z",
            "schemas": {
                "prediction": {
                    "input": {"key1": "value1", "key2": 123},
                    "output": {"result": True, "message": "Success"},
                }
            },
            "container_image_digest": "sha256:abc123def456ghi",
        },
        "hardware": {
            "name": "test hardware name",
            "identifier": params["hardware_identifier"],
        },
        "predictions_count": 2048,
        "overview": "overview of the model",
        "categories": [
            {
                "name": "model category name",
                "slug": "model category slug",
            },
        ],
    }

    call_count = 0

    def request_assertions(request: httpx.Request) -> None:
        nonlocal call_count
        call_count += 1
        json_payload = json.loads(request.content.decode())
        assert json_payload == params

    router = mock_router(
        route=Route(
            method="POST",
            path="/models",
            status_code=200,
            json=expected_json,
        ),
        callback=request_assertions,
    )

    client = Client(
        client_options=ClientOptions(
            headers={"Authorization": "test-token"},
            timeout=httpx.Timeout(15),
            transport=httpx.MockTransport(router.handler),
        )
    )

    if async_flag:
        response = await client.models.async_create(params)
    else:
        response = client.models.create(params)
    assert response == expected_json
    assert call_count == 1


@pytest.mark.asyncio
@pytest.mark.parametrize("async_flag", [True, False])
async def test_update(mock_router: Callable[..., Router], async_flag: bool) -> None:
    test_owner = "test_owner"
    test_model_name = "dummy_model"
    description = "Updated description"
    source_url = "https://example.com/updated_source"
    license_url = "https://license.com/updated_license"
    paper_url = "https://paper.com/updated_paper"
    hardware_identifier = "Updated Hardware identifier"
    overview = "overview of the model"

    expected_json = {
        "id": "123e4567-e89b-12d3-a456-426614174000",
        "name": test_model_name,
        "created_at": "2025-08-01T10:00:00Z",
        "visibility": ModelVisibility.public,
        "description": description,
        "updated_at": "2025-08-15T14:30:00Z",
        "source_url": source_url,
        "license_url": license_url,
        "paper_url": paper_url,
        "owner": {
            "id": "user-001",
            "username": test_owner,
            "avatar_url": "https://example.com/avatar.png",
        },
        "latest_version": {
            "number": 1,
            "release_notes": "Initial release.",
            "created_at": "2025-08-10T09:00:00Z",
            "schemas": {
                "prediction": {
                    "input": {"key1": "value1", "key2": 123},
                    "output": {"result": True, "message": "Success"},
                }
            },
            "container_image_digest": "sha256:abc123def456ghi",
        },
        "hardware": {
            "name": "test hardware name",
            "identifier": hardware_identifier,
        },
        "predictions_count": 2048,
        "overview": "overview of the model",
        "categories": [
            {
                "name": "model category name",
                "slug": "model category slug",
            },
        ],
    }
    payload: UpdateModelParams = {
        "description": description,
        "source_url": source_url,
        "license_url": license_url,
        "paper_url": paper_url,
        "hardware_identifier": hardware_identifier,
        "overview": overview,
        "category_slugs": ["slug-1", "slug-2", "slug-3"],
    }

    call_count = 0

    def request_assertions(request: httpx.Request) -> None:
        nonlocal call_count
        call_count += 1
        json_payload = json.loads(request.content.decode())
        assert json_payload == payload

    router = mock_router(
        route=Route(
            method="PATCH",
            path=f"/models/{test_owner}/{test_model_name}",
            status_code=200,
            json=expected_json,
        ),
        callback=request_assertions,
    )
    client = Client(
        client_options=ClientOptions(
            headers={"Authorization": "test-token"},
            timeout=httpx.Timeout(15),
            transport=httpx.MockTransport(router.handler),
        )
    )
    if async_flag:
        response = await client.models.async_update(
            identifier=f"{test_owner}/{test_model_name}",
            params=payload,
        )
    else:
        response = client.models.update(
            identifier=f"{test_owner}/{test_model_name}",
            params=payload,
        )
    assert response == expected_json
    assert call_count == 1


@pytest.mark.asyncio
@pytest.mark.parametrize("async_flag", [True, False])
async def test_search(mock_router: Callable[..., Router], async_flag: bool) -> None:
    expected_cursor = "abc123"
    expected_page_size = "10"
    expected_query = "dummy_query"
    expected_json = {
        "items": [
            {
                "id": "123e4567-e89b-12d3-a456-426614174000",
                "name": "test_model_name",
                "created_at": "2025-08-01T10:00:00Z",
                "visibility": ModelVisibility.public,
                "description": "A computer vision model.",
                "updated_at": "2025-08-15T14:30:00Z",
                "owner": {
                    "id": "user-001",
                    "username": "test_username",
                    "avatar_url": "https://example.com/avatar.png",
                },
                "hardware": {
                    "name": "test hardware name",
                    "identifier": "test-identifier",
                },
                "predictions_count": 2048,
                "categories": [
                    {
                        "name": "model category name",
                        "slug": "model category slug",
                    },
                ],
            }
        ],
        "next_cursor": "abc123",
        "previous_cursor": None,
        "next_url": "/search/models?cursor=abc123",
        "previous_url": None,
    }

    call_count = 0

    def request_assertions(request: httpx.Request) -> None:
        nonlocal call_count
        call_count += 1
        assert request.url.params.get("cursor") == expected_cursor
        assert request.url.params.get("page_size") == expected_page_size
        assert request.url.params.get("q") == expected_query
        assert set(request.url.params.keys()).issubset({
            "cursor",
            "page_size",
            "q",
        })

    router = mock_router(
        route=Route(
            method="GET",
            path="/search/models",
            status_code=200,
            json=expected_json,
        ),
        callback=request_assertions,
    )
    client = Client(
        client_options=ClientOptions(
            headers={"Authorization": "test-token"},
            timeout=httpx.Timeout(15),
            transport=httpx.MockTransport(router.handler),
        )
    )

    pagiantion_options = PaginationOptions({
        "cursor": expected_cursor,
        "page_size": int(expected_page_size),
    })

    if async_flag:
        response = await client.models.async_search(
            q=expected_query, pagination_options=pagiantion_options
        )
    else:
        response = client.models.search(
            q=expected_query, pagination_options=pagiantion_options
        )

    # expected_json is a dictionary.
    # Although the structure matches the expected fields, static type checkers like mypy
    # may not be able to verify this due to dynamic typing.
    assert response == PaginatedResponse(**expected_json)  # type:ignore[arg-type]
    assert call_count == 1



"""Model Versions module."""

from typing import Optional, TypedDict, cast

from wriftai._resource import Resource
from wriftai.common_types import (
    ModelVersion,
    ModelVersionWithDetails,
    NotRequired,
    Schemas,
)
from wriftai.pagination import PaginatedResponse, PaginationOptions


class CreateModelVersionParams(TypedDict):
    """Parameters for creating a version of a model."""

    release_notes: str
    """Information about changes such as new features, bug fixes, or optimizations
        in this model version."""

    schemas: Schemas
    """Schemas for the model version."""

    container_image_digest: str
    """SHA256 hash digest of the model version's container image."""

    openai_chat_completions: NotRequired[bool]
    """Represents whether this version supports the
    OpenAI compatible chat completions endpoint."""

    openai_embeddings: NotRequired[bool]
    """Represents whether this version supports the
    OpenAI compatible embeddings endpoint."""

    openai_images_generations: NotRequired[bool]
    """Represents whether this version supports the
    OpenAI compatible image generation endpoint."""

    anthropic_messages: NotRequired[bool]
    """Represents whether this version supports the
    Anthropic compatible messages endpoint."""


class ModelVersions(Resource):
    """Resource for operations related to model versions."""

    _ERROR_MSG_INVALID_MODEL_VERSION_IDENTIFIER = (
        "Model Version Identifier must be in owner/name:version-number format."
    )

    def _parse_model_version_identifier(self, identifier: str) -> tuple[str, str, str]:
        """Parses model version identifier string into owner, name, and version number.

        Args:
            identifier: The model version identifier in owner/name:version-number
                format (for example: deepseek-ai/deepseek-r1:1).

        Returns:
            A tuple containing owner, name, and version number of the model.

        Raises:
            ValueError: When the provided model version identifier is not in
                owner/name:version-number format.
        """
        owner, name, number = self._parse_identifier(identifier=identifier)

        if number is None:
            raise ValueError(self._ERROR_MSG_INVALID_MODEL_VERSION_IDENTIFIER)
        else:
            return owner, name, number

    def get(self, identifier: str) -> ModelVersionWithDetails:
        """Get a model version.

        Args:
            identifier: The model version identifier in owner/name:version-number
                format (for example: deepseek-ai/deepseek-r1:1).

        Returns:
            The model version.
        """
        model_owner, model_name, number = self._parse_model_version_identifier(
            identifier=identifier
        )
        response = self._api.request(
            "GET",
            f"{self._MODELS_API_PREFIX}/{model_owner}/{model_name}{self._MODEL_VERSIONS_PATH}/{number}",
        )
        return cast(ModelVersionWithDetails, response)

    async def async_get(self, identifier: str) -> ModelVersionWithDetails:
        """Get a model version.

        Args:
            identifier: The model version identifier in owner/name:version-number
                format (for example: deepseek-ai/deepseek-r1:1).

        Returns:
            The model version.
        """
        model_owner, model_name, number = self._parse_model_version_identifier(
            identifier=identifier
        )
        response = await self._api.async_request(
            "GET",
            f"{self._MODELS_API_PREFIX}/{model_owner}/{model_name}{self._MODEL_VERSIONS_PATH}/{number}",
        )
        return cast(ModelVersionWithDetails, response)

    def list(
        self,
        identifier: str,
        pagination_options: Optional[PaginationOptions] = None,
    ) -> PaginatedResponse[ModelVersion]:
        """List model versions.

        Args:
            identifier: The model identifier in owner/name format (for example:
                deepseek-ai/deepseek-r1).
            pagination_options: Optional settings to control pagination behavior.

        Returns:
            Paginated response containing model versions and navigation metadata.
        """
        model_owner, model_name = self._parse_model_identifier(identifier=identifier)
        path = (
            f"{self._MODELS_API_PREFIX}/{model_owner}"
            f"/{model_name}{self._MODEL_VERSIONS_PATH}"
        )
        response = self._api.request(method="GET", params=pagination_options, path=path)

        # The response will always match the PaginatedResponse structure,
        # but static type checkers may not infer this correctly.
        # Hence, we ignore the argument type warning.
        return PaginatedResponse(**response)  # type:ignore[arg-type]

    async def async_list(
        self,
        identifier: str,
        pagination_options: Optional[PaginationOptions] = None,
    ) -> PaginatedResponse[ModelVersion]:
        """List model versions.

        Args:
            identifier: The model identifier in owner/name format (for example:
                deepseek-ai/deepseek-r1).
            pagination_options: Optional settings to control pagination behavior.

        Returns:
            Paginated response containing model versions and navigation metadata.
        """
        model_owner, model_name = self._parse_model_identifier(identifier=identifier)
        path = (
            f"{self._MODELS_API_PREFIX}/{model_owner}"
            f"/{model_name}{self._MODEL_VERSIONS_PATH}"
        )
        response = await self._api.async_request(
            method="GET", params=pagination_options, path=path
        )
        # The response will always match the PaginatedResponse structure,
        # but static type checkers may not infer this correctly.
        # Hence, we ignore the argument type warning.
        return PaginatedResponse(**response)  # type:ignore[arg-type]

    def delete(self, identifier: str) -> None:
        """Delete a model version.

        Args:
            identifier: The model version identifier in owner/name:version-number
                format (for example: deepseek-ai/deepseek-r1:1).
        """
        model_owner, model_name, number = self._parse_model_version_identifier(
            identifier=identifier
        )
        self._api.request(
            "DELETE",
            f"{self._MODELS_API_PREFIX}/{model_owner}/{model_name}{self._MODEL_VERSIONS_PATH}/{number}",
        )

    async def async_delete(self, identifier: str) -> None:
        """Delete a model version.

        Args:
            identifier: The model version identifier in owner/name:version-number
                format (for example: deepseek-ai/deepseek-r1:1).
        """
        model_owner, model_name, number = self._parse_model_version_identifier(
            identifier=identifier
        )
        await self._api.async_request(
            "DELETE",
            f"{self._MODELS_API_PREFIX}/{model_owner}/{model_name}{self._MODEL_VERSIONS_PATH}/{number}",
        )

    def create(
        self,
        identifier: str,
        options: CreateModelVersionParams,
    ) -> ModelVersionWithDetails:
        """Create a version of a model.

        Args:
            identifier: The model identifier in owner/name format (for example:
                deepseek-ai/deepseek-r1).
            options: Model's version creation parameters.

        Returns:
            The new model version.
        """
        model_owner, model_name = self._parse_model_identifier(identifier=identifier)
        path = (
            f"{self._MODELS_API_PREFIX}/{model_owner}"
            f"/{model_name}{self._MODEL_VERSIONS_PATH}"
        )
        response = self._api.request(
            "POST",
            path,
            # The params matches JsonValue at runtime,
            # but static type checkers may not infer this correctly.
            # Hence, we ignore the argument type warning.
            body=options,  # type:ignore[arg-type]
        )
        return cast(ModelVersionWithDetails, response)

    async def async_create(
        self,
        identifier: str,
        options: CreateModelVersionParams,
    ) -> ModelVersionWithDetails:
        """Create a version of a model.

        Args:
            identifier: The model identifier in owner/name format (for example:
                deepseek-ai/deepseek-r1).
            options: Model's version creation parameters.

        Returns:
            The new model version.
        """
        model_owner, model_name = self._parse_model_identifier(identifier=identifier)
        path = (
            f"{self._MODELS_API_PREFIX}/{model_owner}"
            f"/{model_name}{self._MODEL_VERSIONS_PATH}"
        )
        response = await self._api.async_request(
            "POST",
            path,
            # The params matches JsonValue at runtime,
            # but static type checkers may not infer this correctly.
            # Hence, we ignore the argument type warning.
            body=options,  # type:ignore[arg-type]
        )
        return cast(ModelVersionWithDetails, response)



"""Models module."""

from typing import Optional, TypedDict, cast

from wriftai._resource import Resource
from wriftai.common_types import (
    Model,
    ModelVersionWithDetails,
    ModelVisibility,
    NotRequired,
    SortDirection,
    StrEnum,
)
from wriftai.pagination import PaginatedResponse, PaginationOptions


class ModelWithDetails(Model):
    """Represents a model with details."""

    overview: str | None
    """The overview of the model."""
    latest_version: ModelVersionWithDetails | None
    """The details of the latest version of the model."""
    source_url: str | None
    """Source url from where the model's code can be referenced."""
    license_url: str | None
    """License url where the model's usage is specified."""
    paper_url: str | None
    """Paper url from where research info on the model
        can be found."""


class ModelsSortBy(StrEnum):
    """Enumeration of possible sorting options for querying models."""

    CREATED_AT = "created_at"
    PREDICTIONS_COUNT = "predictions_count"


class ModelPaginationOptions(PaginationOptions):
    """Pagination options for querying models."""

    sort_by: NotRequired[ModelsSortBy]
    """The sorting criteria."""

    sort_direction: NotRequired[SortDirection]
    """The sorting direction."""

    category_slugs: NotRequired[list[str]]
    """The list of category slugs to filter models."""


class UpdateModelParams(TypedDict):
    """Parameters for updating a model."""

    name: NotRequired[str]
    """The name of the model."""
    description: NotRequired[str | None]
    """Description of the model."""
    visibility: NotRequired[ModelVisibility]
    """The visibility of the model."""
    hardware_identifier: NotRequired[str]
    """The identifier of the hardware used by the model."""
    source_url: NotRequired[str | None]
    """Source url from where the model's code can be referenced."""
    license_url: NotRequired[str | None]
    """License url where the model's usage is specified."""
    paper_url: NotRequired[str | None]
    """Paper url from where research info on the model can be
        found."""
    overview: NotRequired[str | None]
    """The overview of the model."""
    category_slugs: NotRequired[list[str]]
    """List of model category slugs."""


class CreateModelParams(TypedDict):
    """Parameters for creating a model."""

    name: str
    """The name of the model."""
    hardware_identifier: str
    """The identifier of the hardware used by the model."""
    visibility: NotRequired[ModelVisibility]
    """The visibility of the model."""
    description: NotRequired[str | None]
    """Description of the model."""
    source_url: NotRequired[str | None]
    """Source url from where the model's code can be referenced."""
    license_url: NotRequired[str | None]
    """License url where the model's usage is specified."""
    paper_url: NotRequired[str | None]
    """Paper url from where research info on the model can be
        found."""
    overview: NotRequired[str | None]
    """The overview of the model."""
    category_slugs: NotRequired[list[str]]
    """List of model category slugs."""


class ModelsResource(Resource):
    """Resource for operations related to models."""

    _SEARCH_MODELS_SUFFIX = "/models"

    def delete(self, identifier: str) -> None:
        """Delete a model.

        Args:
            identifier: The model identifier in owner/name format (for example:
                deepseek-ai/deepseek-r1).
        """
        owner, name = self._parse_model_identifier(identifier=identifier)
        self._api.request("DELETE", f"{self._MODELS_API_PREFIX}/{owner}/{name}")

    async def async_delete(self, identifier: str) -> None:
        """Delete a model.

        Args:
            identifier: The model identifier in owner/name format (for example:
                deepseek-ai/deepseek-r1).
        """
        owner, name = self._parse_model_identifier(identifier=identifier)
        await self._api.async_request(
            "DELETE", f"{self._MODELS_API_PREFIX}/{owner}/{name}"
        )

    def list(
        self,
        pagination_options: Optional[ModelPaginationOptions] = None,
        owner: Optional[str] = None,
    ) -> PaginatedResponse[Model]:
        """List models.

        Args:
            pagination_options: Optional settings to control pagination behavior.
            owner: Username of the model's owner to fetch models for. If None, all
                models are fetched.

        Returns:
            Paginated response containing models and navigation metadata.
        """
        path = self._build_list_path(owner)

        response = self._api.request(method="GET", params=pagination_options, path=path)

        # The response will always match the ModelPaginatedResponse structure,
        # but static type checkers may not infer this correctly.
        # Hence, we ignore the argument type warning.
        return PaginatedResponse(**response)  # type:ignore[arg-type]

    async def async_list(
        self,
        pagination_options: Optional[ModelPaginationOptions] = None,
        owner: Optional[str] = None,
    ) -> PaginatedResponse[Model]:
        """List models.

        Args:
            pagination_options: Optional settings to control pagination behavior.
            owner: Username of the model's owner to fetch models for. If None, all
                models are fetched.

        Returns:
            Paginated response containing models and navigation metadata.
        """
        path = self._build_list_path(owner)

        response = await self._api.async_request(
            method="GET", params=pagination_options, path=path
        )
        # The response will always match the ModelPaginatedResponse structure,
        # but static type checkers may not infer this correctly.
        # Hence, we ignore the argument type warning.
        return PaginatedResponse(**response)  # type:ignore[arg-type]

    def _build_list_path(self, owner: Optional[str] = None) -> str:
        """Construct the API path for listing models.

        Args:
            owner: Username of the model's owner to fetch models for. If None, returns
                a path to fetch all models.

        Returns:
            The constructed API path for listing models.
        """
        return (
            f"{self._MODELS_API_PREFIX}/{owner}" if owner else self._MODELS_API_PREFIX
        )

    def get(self, identifier: str) -> ModelWithDetails:
        """Get a model.

        Args:
            identifier: The model identifier in owner/name format (for example:
                deepseek-ai/deepseek-r1).

        Returns:
            A model object.
        """
        owner, name = self._parse_model_identifier(identifier=identifier)
        response = self._api.request("GET", f"{self._MODELS_API_PREFIX}/{owner}/{name}")

        return cast(ModelWithDetails, response)

    async def async_get(self, identifier: str) -> ModelWithDetails:
        """Get a model.

        Args:
            identifier: The model identifier in owner/name format (for example:
                deepseek-ai/deepseek-r1).

        Returns:
            A model object.
        """
        owner, name = self._parse_model_identifier(identifier=identifier)
        response = await self._api.async_request(
            "GET", f"{self._MODELS_API_PREFIX}/{owner}/{name}"
        )

        return cast(ModelWithDetails, response)

    def create(self, params: CreateModelParams) -> ModelWithDetails:
        """Create a model.

        Args:
            params: Model creation parameters.

        Returns:
            The new model.
        """
        response = self._api.request(
            method="POST",
            path=self._MODELS_API_PREFIX,
            # The params matches JsonValue at runtime,
            # but static type checkers may not infer this correctly.
            # Hence, we ignore the argument type warning.
            body=params,  # type:ignore[arg-type]
        )

        return cast(ModelWithDetails, response)

    async def async_create(self, params: CreateModelParams) -> ModelWithDetails:
        """Create a model.

        Args:
            params: Model creation parameters.

        Returns:
            The new model.
        """
        response = await self._api.async_request(
            method="POST",
            path=self._MODELS_API_PREFIX,
            # The params matches JsonValue at runtime,
            # but static type checkers may not infer this correctly.
            # Hence, we ignore the argument type warning.
            body=params,  # type:ignore[arg-type]
        )

        return cast(ModelWithDetails, response)

    def update(
        self,
        identifier: str,
        params: UpdateModelParams,
    ) -> ModelWithDetails:
        """Update a model.

        Args:
            identifier: The model identifier in owner/name format (for example:
                deepseek-ai/deepseek-r1).
            params: The fields to update.

        Returns:
            The updated model.
        """
        owner, name = self._parse_model_identifier(identifier=identifier)
        response = self._api.request(
            method="PATCH",
            path=f"{self._MODELS_API_PREFIX}/{owner}/{name}",
            # The params matches JsonValue at runtime,
            # but static type checkers may not infer this correctly.
            # Hence, we ignore the argument type warning.
            body=params,  # type: ignore[arg-type]
        )
        return cast(ModelWithDetails, response)

    async def async_update(
        self,
        identifier: str,
        params: UpdateModelParams,
    ) -> ModelWithDetails:
        """Update a model.

        Args:
            identifier: The model identifier in owner/name format (for example:
                deepseek-ai/deepseek-r1).
            params: The fields to update.

        Returns:
            The updated model.
        """
        owner, name = self._parse_model_identifier(identifier=identifier)
        response = await self._api.async_request(
            method="PATCH",
            path=f"{self._MODELS_API_PREFIX}/{owner}/{name}",
            # The params matches JsonValue at runtime,
            # but static type checkers may not infer this correctly.
            # Hence, we ignore the argument type warning.
            body=params,  # type: ignore[arg-type]
        )

        return cast(ModelWithDetails, response)

    def search(
        self,
        q: str,
        pagination_options: Optional[PaginationOptions] = None,
    ) -> PaginatedResponse[Model]:
        """Search models.

        Args:
            q: The search query.
            pagination_options: Optional settings to control pagination behavior.

        Returns:
            Paginated response containing models and navigation metadata.
        """
        params = self._build_search_params(q, pagination_options)

        response = self._api.request(
            method="GET",
            params=params,
            path=f"{self._SEARCH_API_PREFIX}{self._SEARCH_MODELS_SUFFIX}",
        )
        # The response will always match the PaginatedResponse structure,
        # but static type checkers may not infer this correctly.
        # Hence, we ignore the argument type warning.
        return PaginatedResponse(**response)  # type:ignore[arg-type]

    async def async_search(
        self,
        q: str,
        pagination_options: Optional[PaginationOptions] = None,
    ) -> PaginatedResponse[Model]:
        """Search models.

        Args:
            q: The search query.
            pagination_options: Optional settings to control pagintation behavior.

        Returns:
            Paginated response containing models and navigation metadata.
        """
        params = self._build_search_params(q, pagination_options)

        response = await self._api.async_request(
            method="GET",
            params=params,
            path=f"{self._SEARCH_API_PREFIX}{self._SEARCH_MODELS_SUFFIX}",
        )
        # The response will always match the PaginatedResponse structure,
        # but static type checkers may not infer this correctly.
        # Hence, we ignore the argument type warning.
        return PaginatedResponse(**response)  # type:ignore[arg-type]



"""Resource module."""

from abc import ABC
from collections.abc import Mapping
from typing import Any, Optional

from wriftai.api import API
from wriftai.pagination import PaginationOptions


class Resource(ABC):
    """Abstract base class for API resources."""

    _api: API
    _MODELS_API_PREFIX = "/models"
    _SEARCH_API_PREFIX = "/search"
    _MODEL_VERSIONS_PATH = "/versions"
    _ERROR_MSG_INVALID_IDENTIFIER = (
        "Identifier must be in either owner/name or owner/name:version-number format."
    )
    _ERROR_MSG_INVALID_MODEL_IDENTIFIER = "Model Identifier must in owner/name format."

    def __init__(self, api: API) -> None:
        """Initializes the Resource with an API instance.

        Args:
            api: An instance of the API class.
        """
        self._api = api

    def _build_search_params(
        self, q: str, pagination_options: Optional[PaginationOptions] = None
    ) -> Mapping[str, Any]:
        """Build search parameters.

        Args:
            q: The search query.
            pagination_options: Optional settings to control pagination behavior.

        Returns:
            Parameters for searching.
        """
        params = {**pagination_options} if pagination_options else {}
        params["q"] = q

        return params

    def _parse_identifier(self, identifier: str) -> tuple[str, str, str | None]:
        """Parses model identifier string into owner, name, and version number.

        Args:
            identifier: The identifier in either owner/name or
                owner/name:version-number format (for example:
                deepseek-ai/deepseek-r1 or deepseek-ai/deepseek-r1:1).

        Returns:
            A tuple containing owner, name, and version number of the model.

        Raises:
            ValueError: When the provided identifier is not in owner/name or
                owner/name:version-number format.
        """
        owner, sep, name_part = identifier.partition("/")
        if not (sep and owner and name_part):
            raise ValueError(self._ERROR_MSG_INVALID_IDENTIFIER)

        name, sep, version_number = name_part.partition(":")
        if not name or (sep and not version_number):
            raise ValueError(self._ERROR_MSG_INVALID_IDENTIFIER)

        return owner, name, version_number or None

    def _parse_model_identifier(self, identifier: str) -> tuple[str, str]:
        """Parses model identifier string into owner and name.

        Args:
            identifier: The model identifier in owner/name format (for example:
                deepseek-ai/deepseek-r1).

        Returns:
            A tuple containing owner and name of the model.

        Raises:
            ValueError: When the provided model identifier is not in owner/name format.
        """
        owner, name, number = self._parse_identifier(identifier=identifier)

        if number is not None:
            raise ValueError(self._ERROR_MSG_INVALID_MODEL_IDENTIFIER)
        else:
            return owner, name
